=== modified file '.bzrignore'
--- .bzrignore	2010-04-22 08:52:59 +0000
+++ .bzrignore	2011-03-10 03:54:45 +0000
@@ -9,3 +9,4 @@
 loggerhead-memprofile
 ./docs/_build/
 tags
+loggerhead/static/downloads/*

=== added file 'HACKING'
--- HACKING	1970-01-01 00:00:00 +0000
+++ HACKING	2011-02-10 23:05:31 +0000
@@ -0,0 +1,59 @@
+Loggerhead
+==========
+
+Overview
+--------
+
+This document attempts to give some hints for people that are wanting to work
+on Loggerhead.
+
+
+Testing
+-------
+
+You can run the loggerhead test suite as a bzr plugin. To run just the
+loggerhead tests::
+
+  bzr selftest -s bp.loggerhead
+
+
+Load Testing
+------------
+
+As a web service, Loggerhead will often be hit by multiple requests. We want
+to make sure that loggerhead can scale with many requests, without performing
+poorly or crashing under the load.
+
+There is a command ``bzr load-test-loggerhead`` that can be run to stress
+loggerhead. A script is given, describing what requests to make, against what
+URLs, and for what level of parallel activity.
+
+
+Load Testing Multiple Instances
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+One way that Launchpad provides both high availability and performance scaling
+is by running multiple instances of loggerhead, serving the same content. A
+proxy is then used to load balance the requests. This also allows us to shut
+down one instance for upgrading, without interupting service (requests are
+just routed to the other instance).
+
+However, multiple processes poses an even greater risk that caches will
+conflict. As such, it is useful to test that changes don't introduce coherency
+issues at load. ``bzr load-test-loggerhead`` can be configured with a script
+that will make requests against multiple loggerhead instances concurrently.
+
+To run multiple instances, it is often sufficient to just spawn multiple
+servers on different ports. For example::
+
+  $ bzr serve --http --port=8080 &
+  $ bzr serve --http --port=8081 &
+
+There is a simple example script already in the source tree::
+
+  $ bzr load-test-loggerhead load_test_scripts/multiple_instances.script
+
+
+
+.. vim: ft=rst tw=78
+

=== modified file 'NEWS'
--- NEWS	2010-12-01 08:21:27 +0000
+++ NEWS	2011-03-02 14:07:21 +0000
@@ -1,6 +1,25 @@
 What's changed in loggerhead?
 =============================
 
+1.19 [???]
+----------------
+
+    - Add ``bzr load-test-loggerhead`` as a way to make sure loggerhead can
+      handle concurrent requests, etc. Scripts can be written that spawn
+      multiple threads, and issue concurrent requests.
+      (John Arbash Meinel)
+
+    - If we get a HEAD request, there is no reason to expand the template, we
+      shouldn't be returning body content anyway.
+      (John Arbash Meinel, #716201, #716217)
+
+    - Merge the pqm changes back into trunk, after trunk was reverted to an old
+      revision. (John Arbash Meinel, #716152)
+
+    - The json module is no longer claimed to be supported as alternative for 
+      simplejson. (Jelmer Vernooij, #586611)
+
+
 1.18 [10Nov2010]
 ----------------
 

=== modified file '__init__.py'
--- __init__.py	2010-11-11 02:48:30 +0000
+++ __init__.py	2011-02-10 01:57:32 +0000
@@ -1,4 +1,4 @@
-# Copyright 2009, 2010 Canonical Ltd
+# Copyright 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,21 +38,11 @@
 if __name__ == 'bzrlib.plugins.loggerhead':
     import bzrlib
     from bzrlib.api import require_any_api
+    from bzrlib import commands
 
     require_any_api(bzrlib, bzr_compatible_versions)
 
-    # NB: Normally plugins should lazily load almost everything, but this
-    # seems reasonable to have in-line here: bzrlib.commands and options are
-    # normally loaded, and the rest of loggerhead won't be loaded until serve
-    # --http is run.
-
-    # transport_server_registry was added in bzr 1.16. When we drop support for
-    # older releases, we can remove the code to override cmd_serve.
-
-    try:
-        from bzrlib.transport import transport_server_registry
-    except ImportError:
-        transport_server_registry = None
+    from bzrlib.transport import transport_server_registry
 
     DEFAULT_HOST = '0.0.0.0'
     DEFAULT_PORT = 8080
@@ -70,7 +60,6 @@
         logging.getLogger('simpleTAL').addHandler(handler)
         logging.getLogger('simpleTALES').addHandler(handler)
 
-
     def _ensure_loggerhead_path():
         """Ensure that you can 'import loggerhead' and get the root."""
         # loggerhead internal code will try to 'import loggerhead', so
@@ -103,41 +92,27 @@
         app = HTTPExceptionHandler(app)
         serve(app, host=host, port=port)
 
-    if transport_server_registry is not None:
-        transport_server_registry.register('http', serve_http, help=HELP)
-    else:
-        import bzrlib.builtins
-        from bzrlib.commands import get_cmd_object, register_command
-        from bzrlib.option import Option
-
-        _original_command = get_cmd_object('serve')
-
-        class cmd_serve(bzrlib.builtins.cmd_serve):
-            __doc__ = _original_command.__doc__
-
-            takes_options = _original_command.takes_options + [
-                Option('http', help=HELP)]
-
-            def run(self, *args, **kw):
-                if 'http' in kw:
-                    from bzrlib.transport import get_transport
-                    allow_writes = kw.get('allow_writes', False)
-                    path = kw.get('directory', '.')
-                    port = kw.get('port', DEFAULT_PORT)
-                    # port might be an int already...
-                    if isinstance(port, basestring) and ':' in port:
-                        host, port = port.split(':')
-                    else:
-                        host = DEFAULT_HOST
-                    if allow_writes:
-                        transport = get_transport(path)
-                    else:
-                        transport = get_transport('readonly+' + path)
-                    serve_http(transport, host, port)
-                else:
-                    super(cmd_serve, self).run(*args, **kw)
-
-        register_command(cmd_serve)
+    transport_server_registry.register('http', serve_http, help=HELP)
+
+    class cmd_load_test_loggerhead(commands.Command):
+        """Run a load test against a live loggerhead instance.
+
+        Pass in the name of a script file to run. See loggerhead/load_test.py
+        for a description of the file format.
+        """
+
+        takes_args = ["filename"]
+
+        def run(self, filename):
+            from bzrlib.plugins.loggerhead.loggerhead import load_test
+            script = load_test.run_script(filename)
+            for thread_id in sorted(script._threads):
+                worker = script._threads[thread_id][0]
+                for url, success, time in worker.stats:
+                    self.outf.write(' %5.3fs %s %s\n'
+                                    % (time, str(success)[0], url))
+
+    commands.register_command(cmd_load_test_loggerhead)
 
     def load_tests(standard_tests, module, loader):
         _ensure_loggerhead_path()

=== added directory 'load_test_scripts'
=== added file 'load_test_scripts/multiple_instances.script'
--- load_test_scripts/multiple_instances.script	1970-01-01 00:00:00 +0000
+++ load_test_scripts/multiple_instances.script	2011-02-10 01:00:32 +0000
@@ -0,0 +1,19 @@
+{
+    "comment": "Connect to multiple loggerhead instances and make requests on each. One should be on :8080, one should be on :8081. Multiple threads will place requests on each.",
+    "parameters": {"base_url": "http://localhost"},
+    "requests": [
+        {"thread": "1", "relpath": ":8080/changes"},
+        {"thread": "2", "relpath": ":8080/files"},
+        {"thread": "3", "relpath": ":8081/files"},
+        {"thread": "4", "relpath": ":8081/changes"},
+        {"thread": "1", "relpath": ":8080/changes"},
+        {"thread": "2", "relpath": ":8080/files"},
+        {"thread": "3", "relpath": ":8081/files"},
+        {"thread": "4", "relpath": ":8081/changes"},
+        {"thread": "1", "relpath": ":8080/changes"},
+        {"thread": "2", "relpath": ":8080/files"},
+        {"thread": "3", "relpath": ":8081/files"},
+        {"thread": "4", "relpath": ":8081/changes"}
+    ]
+}
+

=== added file 'load_test_scripts/simple.script'
--- load_test_scripts/simple.script	1970-01-01 00:00:00 +0000
+++ load_test_scripts/simple.script	2011-02-10 01:00:32 +0000
@@ -0,0 +1,9 @@
+{
+    "comment": "A fairly trivial load test script. It just loads the main two pages from a loggerhead install running directly on a branch.",
+    "parameters": {"base_url": "http://localhost:8080"},
+    "requests": [
+        {"relpath": "/changes"},
+        {"relpath": "/files"}
+    ]
+}
+

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2011-02-09 22:29:57 +0000
+++ loggerhead/apps/branch.py	2011-03-10 03:54:45 +0000
@@ -33,7 +33,7 @@
 from loggerhead.controllers.atom_ui import AtomUI
 from loggerhead.controllers.changelog_ui import ChangeLogUI
 from loggerhead.controllers.diff_ui import DiffUI
-from loggerhead.controllers.download_ui import DownloadUI
+from loggerhead.controllers.download_ui import DownloadUI, DownloadTarballUI
 from loggerhead.controllers.filediff_ui import FileDiffUI
 from loggerhead.controllers.inventory_ui import InventoryUI
 from loggerhead.controllers.revision_ui import RevisionUI
@@ -49,7 +49,7 @@
 
     def __init__(self, branch, friendly_name=None, config={},
                  graph_cache=None, branch_link=None, is_root=False,
-                 served_url=_DEFAULT, use_cdn=False):
+                 served_url=_DEFAULT, use_cdn=False, export_tarballs=True):
         self.branch = branch
         self._config = config
         self.friendly_name = friendly_name
@@ -61,6 +61,7 @@
         self.is_root = is_root
         self.served_url = served_url
         self.use_cdn = use_cdn
+        self.export_tarballs = export_tarballs
 
     def get_history(self):
         file_cache = None
@@ -126,6 +127,7 @@
         'revision': RevisionUI,
         'search': SearchUI,
         'view': ViewUI,
+        'tarball': DownloadTarballUI,
         }
 
     def last_updated(self):
@@ -171,10 +173,14 @@
         try:
             try:
                 c = cls(self, self.get_history)
-                return c(environ, start_response)
+                to_ret = c(environ, start_response)
             except:
                 environ['exc_info'] = sys.exc_info()
                 environ['branch'] = self
                 raise
+            if type(to_ret) == type(httpexceptions.HTTPSeeOther('/')):
+                raise to_ret
+            else:
+                return to_ret
         finally:
             self.branch.unlock()

=== modified file 'loggerhead/apps/config.py'
--- loggerhead/apps/config.py	2010-05-05 18:35:35 +0000
+++ loggerhead/apps/config.py	2011-02-11 22:10:56 +0000
@@ -10,7 +10,10 @@
 
 import bzrlib.lru_cache
 
-from bzrlib.util.configobj.configobj import ConfigObj
+try:
+    from bzrlib.util.configobj.configobj import ConfigObj
+except ImportError:
+    from configobj import ConfigObj
 
 from paste.request import path_info_pop
 from paste import httpexceptions

=== modified file 'loggerhead/config.py'
--- loggerhead/config.py	2010-04-13 23:35:01 +0000
+++ loggerhead/config.py	2011-03-10 03:54:45 +0000
@@ -36,6 +36,7 @@
         use_cdn=False,
         sql_dir=None,
         allow_writes=False,
+        export_tarballs=True,
         )
     parser.add_option("--user-dirs", action="store_true",
                       help="Serve user directories as ~user.")
@@ -69,6 +70,8 @@
                       help="The directory to place the SQL cache in")
     parser.add_option("--allow-writes", action="store_true",
                       help="Allow writing to the Bazaar server.")
+    parser.add_option("--export-tarballs", action="store_true",
+                      help="Allow exporting revisions to tarballs.")
     return parser
 
 

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2011-01-20 07:28:21 +0000
+++ loggerhead/controllers/__init__.py	2011-03-10 03:54:45 +0000
@@ -20,7 +20,7 @@
 import bzrlib.errors
 import time
 
-from paste.httpexceptions import HTTPNotFound
+from paste.httpexceptions import HTTPNotFound, HTTPSeeOther
 from paste.request import path_info_pop, parse_querystring
 
 from loggerhead import util
@@ -93,12 +93,17 @@
         headers = {}
 
         vals.update(self.get_values(path, kwargs, headers))
+        if 'download' in vals:
+            return HTTPSeeOther(vals['download'])
 
         self.log.info('Getting information for %s: %r secs' % (
             self.__class__.__name__, time.time() - z))
         if 'Content-Type' not in headers:
             headers['Content-Type'] = 'text/html'
         writer = start_response("200 OK", headers.items())
+        if environ.get('REQUEST_METHOD') == 'HEAD':
+            # No content for a HEAD request
+            return []
         template = load_template(self.template_path)
         z = time.time()
         w = BufferingWriter(writer, 8192)

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2010-04-22 08:52:59 +0000
+++ loggerhead/controllers/changelog_ui.py	2011-03-02 14:07:21 +0000
@@ -20,10 +20,7 @@
 
 import urllib
 
-try:
-    import simplejson
-except ImportError:
-    import json as simplejson
+import simplejson
 
 from paste.httpexceptions import HTTPServerError
 

=== modified file 'loggerhead/controllers/download_ui.py'
--- loggerhead/controllers/download_ui.py	2010-05-05 19:03:40 +0000
+++ loggerhead/controllers/download_ui.py	2011-03-10 03:54:45 +0000
@@ -19,6 +19,7 @@
 
 import logging
 import mimetypes
+import os
 import urllib
 
 from paste import httpexceptions
@@ -67,3 +68,29 @@
             ]
         start_response('200 OK', headers)
         return [content]
+
+class DownloadTarballUI(TemplatedBranchView):
+    """Download a revno as a tarball or zip file."""
+    ext = 'tar.gz'
+    dest = os.path.join(os.getcwd(), 'loggerhead/static/downloads/')
+    download_dir = '/static/downloads/'
+ 
+    def get_values(self, path, kwargs, headers):
+        """Return a URL to a tarball.
+
+        In the form of: /tarball/revno_or_revid."""
+        history = self._history
+        if len(self.args):
+            revid = history.fix_revid(self.args[0])
+        else:
+            revid = self.get_revid()
+        revno = history.get_revno(revid)
+        filename = '%s_%s.%s' % (history._branch_nick, revno, self.ext)
+        rpath = os.getcwd()+ '/loggerhead/static/downloads/' + filename
+        if not os.path.exists(rpath):
+            history.export(revid, rpath)
+        if self._branch.export_tarballs:
+            return {'download': '/static/downloads/' + filename}
+        else:
+            # redirect to the home page, the user is cheating :)
+            return {'download': '/'}

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2009-08-04 04:08:03 +0000
+++ loggerhead/controllers/revision_ui.py	2011-03-10 03:54:45 +0000
@@ -17,10 +17,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-try:
-    import simplejson
-except ImportError:
-    import json as simplejson
+import simplejson
 import urllib
 
 from paste.httpexceptions import HTTPServerError
@@ -113,7 +110,7 @@
                 self._branch.friendly_name,
                 self._branch.is_root,
                 'changes'))
-
+        can_export = self._branch.export_tarballs
         return {
             'branch': self._branch,
             'revid': revid,
@@ -135,4 +132,5 @@
             'compare_revid': compare_revid,
             'url': self._branch.context_url,
             'directory_breadcrumbs': directory_breadcrumbs,
+            'can_export': can_export,
         }

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2010-05-10 13:48:25 +0000
+++ loggerhead/history.py	2011-05-30 22:57:48 +0000
@@ -38,12 +38,14 @@
 import bzrlib.branch
 import bzrlib.delta
 import bzrlib.errors
+import bzrlib.export
 import bzrlib.foreign
 import bzrlib.revision
 
 from loggerhead import search
 from loggerhead import util
 from loggerhead.wholehistory import compute_whole_history_data
+from bzrlib.export.tar_exporter import export_tarball
 
 
 def is_branch(folder):
@@ -760,3 +762,9 @@
             removed=sorted(reporter.removed, key=lambda x: x.filename),
             modified=sorted(reporter.modified, key=lambda x: x.filename),
             text_changes=sorted(reporter.text_changes, key=lambda x: x.filename))
+
+    def export(self, revid, ball):
+        """Export a tarball of a revision."""
+        rev_tree = self._branch.repository.revision_tree(revid)
+        export_tarball(rev_tree, ball, None)
+        return ball

=== added file 'loggerhead/load_test.py'
--- loggerhead/load_test.py	1970-01-01 00:00:00 +0000
+++ loggerhead/load_test.py	2011-03-02 14:07:21 +0000
@@ -0,0 +1,230 @@
+# Copyright 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Code to do some load testing against a loggerhead instance.
+
+This is basically meant to take a list of actions to take, and run it against a
+real host, and see how the results respond.::
+
+    {"parameters": {
+         "base_url": "http://localhost:8080",
+     },
+     "requests": [
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"}
+     ],
+    }
+
+All threads have a Queue length of 1. When a third request for a given thread
+is seen, no more requests are queued until that thread finishes its current
+job. So this results in all requests being issued sequentially::
+
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"}
+
+While this would cause all requests to be sent in parallel:
+
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "2", "relpath": "/changes"},
+        {"thread": "3", "relpath": "/changes"},
+        {"thread": "4", "relpath": "/changes"}
+
+This should keep 2 threads pipelined with activity, as long as they finish in
+approximately the same speed. We'll start the first thread running, and the
+second thread, and queue up both with a second request once the first finishes.
+When we get to the third request for thread "1", we block on queuing up more
+work until the first thread 1 request has finished.
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "2", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "2", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "2", "relpath": "/changes"}
+
+There is not currently a way to say "run all these requests keeping exactly 2
+threads active". Though if you know the load pattern, you could approximate
+this.
+"""
+
+import threading
+import time
+import Queue
+
+import simplejson
+
+from bzrlib import (
+    errors,
+    transport,
+    urlutils,
+    )
+
+# This code will be doing multi-threaded requests against bzrlib.transport
+# code. We want to make sure to load everything ahead of time, so we don't get
+# lazy-import failures
+_ = transport.get_transport('http://example.com')
+
+
+class RequestDescription(object):
+    """Describes info about a request."""
+
+    def __init__(self, descrip_dict):
+        self.thread = descrip_dict.get('thread', '1')
+        self.relpath = descrip_dict['relpath']
+
+
+class RequestWorker(object):
+    """Process requests in a worker thread."""
+
+    _timer = time.time
+
+    def __init__(self, identifier, blocking_time=1.0, _queue_size=1):
+        self.identifier = identifier
+        self.queue = Queue.Queue(_queue_size)
+        self.start_time = self.end_time = None
+        self.stats = []
+        self.blocking_time = blocking_time
+
+    def step_next(self):
+        url = self.queue.get(True, self.blocking_time)
+        if url == '<noop>':
+            # This is usually an indicator that we want to stop, so just skip
+            # this one.
+            self.queue.task_done()
+            return
+        self.start_time = self._timer()
+        success = self.process(url)
+        self.end_time = self._timer()
+        self.update_stats(url, success)
+        self.queue.task_done()
+
+    def run(self, stop_event):
+        while not stop_event.isSet():
+            try:
+                self.step_next()
+            except Queue.Empty:
+                pass
+
+    def process(self, url):
+        base, path = urlutils.split(url)
+        t = transport.get_transport(base)
+        try:
+            # TODO: We should probably look into using some part of
+            #       blocking_timeout to decide when to stop trying to read
+            #       content
+            content = t.get_bytes(path)
+        except (errors.TransportError, errors.NoSuchFile):
+            return False
+        return True
+
+    def update_stats(self, url, success):
+        self.stats.append((url, success, self.end_time - self.start_time))
+
+
+class ActionScript(object):
+    """This tracks the actions that we want to perform."""
+
+    _worker_class = RequestWorker
+    _default_base_url = 'http://localhost:8080'
+    _default_blocking_timeout = 60.0
+
+    def __init__(self):
+        self.base_url = self._default_base_url
+        self.blocking_timeout = self._default_blocking_timeout
+        self._requests = []
+        self._threads = {}
+        self.stop_event = threading.Event()
+
+    @classmethod
+    def parse(cls, content):
+        script = cls()
+        json_dict = simplejson.loads(content)
+        if 'parameters' not in json_dict:
+            raise ValueError('Missing "parameters" section')
+        if 'requests' not in json_dict:
+            raise ValueError('Missing "requests" section')
+        param_dict = json_dict['parameters']
+        request_list = json_dict['requests']
+        base_url = param_dict.get('base_url', None)
+        if base_url is not None:
+            script.base_url = base_url
+        blocking_timeout = param_dict.get('blocking_timeout', None)
+        if blocking_timeout is not None:
+            script.blocking_timeout = blocking_timeout
+        for request_dict in request_list:
+            script.add_request(request_dict)
+        return script
+
+    def add_request(self, request_dict):
+        request = RequestDescription(request_dict)
+        self._requests.append(request)
+
+    def _get_worker(self, thread_id):
+        if thread_id in self._threads:
+            return self._threads[thread_id][0]
+        handler = self._worker_class(thread_id,
+                                     blocking_time=self.blocking_timeout/10.0)
+
+        t = threading.Thread(target=handler.run, args=(self.stop_event,),
+                             name='Thread-%s' % (thread_id,))
+        self._threads[thread_id] = (handler, t)
+        t.start()
+        return handler
+
+    def finish_queues(self):
+        """Wait for all queues of all children to finish."""
+        for h, t in self._threads.itervalues():
+            h.queue.join()
+
+    def stop_and_join(self):
+        """Stop all running workers, and return.
+
+        This will stop even if workers still have work items.
+        """
+        self.stop_event.set()
+        for h, t in self._threads.itervalues():
+            # Signal the queue that it should stop blocking, we don't have to
+            # wait for the queue to empty, because we may see stop_event before
+            # we see the <noop>
+            h.queue.put('<noop>')
+            # And join the controlling thread
+            for i in range(10):
+                t.join(self.blocking_timeout / 10.0)
+                if not t.isAlive():
+                    break
+
+    def _full_url(self, relpath):
+        return self.base_url + relpath
+
+    def run(self):
+        self.stop_event.clear()
+        for request in self._requests:
+            full_url = self._full_url(request.relpath)
+            worker = self._get_worker(request.thread)
+            worker.queue.put(full_url, True, self.blocking_timeout)
+        self.finish_queues()
+        self.stop_and_join()
+
+
+def run_script(filename):
+    with open(filename, 'rb') as f:
+        content = f.read()
+    script = ActionScript.parse(content)
+    script.run()
+    return script

=== modified file 'loggerhead/static/css/diff.css'
--- loggerhead/static/css/diff.css	2010-01-11 03:26:44 +0000
+++ loggerhead/static/css/diff.css	2011-02-18 05:41:17 +0000
@@ -1,164 +1,113 @@
 /*Info Box*/
 .infoContainer {
-    width:970px;
-    padding:10px 10px 0px 10px;
-    margin:0px 0px 10px 0px;
-    color:#666;
-    background:#fcfcfc url(../images/bg_infobox.gif) repeat-y top right;
-    border:1px solid #b7b7b7;
-}
+    width: 970px;
+    padding: 10px 10px 0 10px;
+    margin: 0 0 10px 0;
+    color: #666;
+    background: #fcfcfc url(../images/bg_infobox.gif) repeat-y top right;
+    border: 1px solid #b7b7b7;
+    }
 ul#list-files {
-    float:right;
-    width:460px;
-    list-style:none;
-    margin:0px;
-    padding:0px;
-    font:normal 11px/normal sans-serif;
-    color:#000;
-}
+    float: right;
+    width: 460px;
+    font-size: 77%;
+    color: #000;
+    }
 ul#list-files li {
-    width:440px;
-    padding:2px 0px 2px 20px;
-    margin:0px 0px 2px 8px;
-    background:url(../images/ico_file.gif) 2px 0px no-repeat;
-}
+    width: 440px;
+    padding: 2px 0 2px 20px;
+    margin: 0 0 2px 8px;
+    background: url(../images/ico_file.gif) 2px 0 no-repeat;
+    }
 ul#list-files li.desc {
     margin-left: 0;
-    background:url(../images/ico_description.gif) 0px 0px no-repeat;
-}
+    background: url(../images/ico_description.gif) 0 0 no-repeat;
+    }
 #infTxt {
-    float:left;
-    width:493px;
-    margin:0px 0px 5px 0px;
-}
-.information, .committer, .timer, .mfrom, .mto, .revid, .bug, .tags, .foreign-svn, .foreign-git, .foreign-hg {
-    clear:both;
-    margin:0px;
-    padding:0px 0px 2px 18px;
-    font:normal 11px/1.3 sans-serif;
-    color:#666;
-    background:url(../images/ico_description.gif) no-repeat top left;
-}
-.revid {
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-}
-.tags {
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-    background:url(../images/ico_tag.gif) no-repeat top left;
-}
-.committer, .mfrom, .mto {
-    color:#000;
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-    float:left;
-    display:inline;
-    background:url(../images/ico_committer.gif) no-repeat 2px 0px;
-}
-.timer {
-    margin:0px 10px 5px 0px;
-    padding:1px 0px 0px 18px;
-    float:left;
-    clear:none;
-    display:inline;
-    background:url(../images/ico_time.gif) no-repeat top left;
-}
-.mfrom {
-    background:url(../images/ico_mergefrom.gif) no-repeat top left;
-}
-.mto {
-    background:url(../images/ico_mergeto.gif) no-repeat top left;
-}
-.bug {
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-    float:left;
-    clear:none;
-    display:inline;
-    background:url(../images/ico_bug.png) no-repeat top left;
-}
-/**/
-#loggerheadCont p.expand, #loggerheadCont p.expand a, #loggerheadCont p.expand a:link, #loggerheadCont p.expand a:visited, #loggerheadCont p.expand a:hover {
-    color:#0f6b97;
-    text-decoration:none;
-    margin-right:10px;
-    float:left;
-}
-#loggerheadCont p.codin, #loggerheadCont p.codin a, #loggerheadCont p.codin a:link, #loggerheadCont p.codin a:visited, #loggerheadCont p.codin a:hover {
-    color:#000;
-    font-size:12px;
-    text-decoration:none;
-    margin-right:10px;
-    float:left;
-}
+    float: left;
+    width: 493px;
+    margin: 0 0 5px 0;
+    font-size:  77%;
+    color: #666;
+    }
 /*Diff Boxes*/
 .diffBox {
-    width:100%;
-    padding:6px 0%;
-    background:#eef6fa;
-    border:1px solid #d2d2d2;
-    margin:10px 0px 0px 0px;
-}
+    width: 100%;
+    padding: 6px 0;
+    background: #eef6fa;
+    border: 1px solid #d2d2d2;
+    margin: 10px 0 0 0;
+    }
 .diffBox img {
-    vertical-align:middle;
-}
-.diffBox a, .diffBox a:link, .diffBox a:visited, .diffBox a:hover {
-    text-decoration:none;
-}
+    vertical-align: middle;
+    }
+.diffBox a {
+    text-decoration: none;
+    }
 .diff {
-    margin: 2em 0 2em 0;
-}
+    margin: 2em 0;
+    }
 .diffinfo {
     overflow: hidden;
-}
+    }
 .lineNumber {
-    width:2%;
-    border-right:1px solid #d5d5d5;
-    border-left:1px solid #d5d5d5;
-    background:#f8fbfd;
-    text-align:right;
-}
+    width: 2%;
+    border-width: 0 1px;
+    border-style: solid;
+    border-color: #d5d5d5;
+    background: #f8fbfd;
+    text-align: right;
+    }
 .unified .lineNumber.second {
-    border-left:0px;
-}
-.code, .lineNumber {
-    float:left;
-    padding:0px 1%;
-    color:#000;
-    font:normal 12px/1.8 monospace;
-}
-.blank {
-    float: left;
-    padding:0px 1%;
-}
-.unified .code { width:89%; }
-.sbs .code, .blank {
-    width:43%;
+    border-left: 0;
+    }
+.code,
+.lineNumber {
+    float: left;
+    padding: 0 1%;
+    color: #000;
+    font-size: 85%;
+    font-family: 'UbuntuBeta Mono', 'Ubuntu Mono', monospace;
+    line-height: 180%;
+    }
+.blank {
+    float: left;
+    padding: 0 1%;
+    }
+.unified .code {
+    width: 89%;
+    }
+.sbs .code,
+.blank {
+    width: 43%;
     overflow: hidden;
-}
+    }
 .unified .insert-row .lineNumber.first {
-    background:#c9ffc9;
-}
+    background: #c9ffc9;
+    }
 .unified .delete-row .lineNumber.second {
-    background:#ffbfbf;
-}
-.code.insert, .insert-row .lineNumber.second, .both-row .lineNumber.second {
-    background:#92ed92;
-}
-.code.delete, .delete-row .lineNumber.first, .both-row .lineNumber.first {
-    background:#ff7f7f;
-}
+    background: #ffbfbf;
+    }
+.code.insert,
+.insert-row .lineNumber.second,
+.both-row .lineNumber.second {
+    background: #92ed92;
+    }
+.code.delete,
+.delete-row .lineNumber.first,
+.both-row .lineNumber.first {
+    background: #ff7f7f;
+    }
 .separate {
     height: 2em;
     background: #e3e3e3;
-}
+    }
 ul#pages {
-    margin-top:10px;
-}
+    margin-top: 10px;
+    }
 /*Foot Links*/
 #loggerheadCont p.lpandbzr {
-    float:left;
-    padding:3px 0px 0px 0px;
-    margin-top:10px;
-}
+    float: left;
+    padding: 3px 0 0 0;
+    margin-top: 10px;
+    }

=== modified file 'loggerhead/static/css/files.css'
--- loggerhead/static/css/files.css	2009-06-17 20:28:20 +0000
+++ loggerhead/static/css/files.css	2011-02-18 05:41:17 +0000
@@ -1,50 +1,5 @@
-/*Info Box*/
-.information, .committer, .timer, .mfrom, .mto, .revid, .bug, .tags {
-	float:left;
-	margin:0px 10px 5px 0px;
-	padding:0px 0px 2px 18px;
-	font:normal 11px/1.3 sans-serif;
-	color:#666;
-	background:url(../images/ico_description.gif) no-repeat top left;
-}
-.revid {
-    float:left;
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-}
-.tags {
-    float:left;
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-    background:url(../images/ico_tag.gif) no-repeat top left;
-}
-.comiter, .mfrom, .mto {
-	color:#000;
-	padding:1px 0px 0px 18px;
-	margin:0px 10px 5px 0px;
-	float:left;
-	display:inline;
-	background:url(../images/ico_committer.gif) no-repeat 2px 0px;
-}
-.timer {
-	margin:0px 10px 5px 0px;
-	padding:1px 0px 0px 18px;
-	float:left;
-	clear:none;
-	display:inline;
-	background:url(../images/ico_time.gif) no-repeat top left;
-}
-.mfrom {
-	background:url(../images/ico_mergefrom.gif) no-repeat top left;
-}
-.mto {
-	background:url(../images/ico_mergetop.gif) no-repeat top left;
-}
-.information {
-    clear:both;
-    width:100%;
-    position:relative;
-}
-.bug {
-	background:url(../images/ico_bug.png) no-repeat top left;
-}
+#infTxt {
+    font-size: 77%;
+    color: #666;
+    margin-bottom: 10px;
+    }

=== modified file 'loggerhead/static/css/global.css'
--- loggerhead/static/css/global.css	2010-04-23 01:30:29 +0000
+++ loggerhead/static/css/global.css	2011-02-24 00:44:09 +0000
@@ -1,354 +1,345 @@
+/*
+    CSS Structure
+    - Reset and Generic Styles
+    - Header & Navigation
+        - Loggerhead Menu Tabs
+        - Sub-menu Tabs
+        - Breadcrumbs
+    - Search & RSS 
+    - General Content
+    - Log Table
+    - Revision List
+    - Pagination
+    - Footer
+*/
+
+
+/* =========================
+   Reset and Generic Styles
+*/
+body, img, form, tr, ul, li, h1, h2, h3, h4, p, table {
+    padding: 0;
+    margin: 0;
+    }
+body, html, table {
+    font-size: 93%;
+    }
 body {
-	padding:0px;
-	margin:0px;
-	font:normal 12px/normal sans-serif;
-	color:#666;
-	background:#fff;
-}
-a, a:link, a:visited, a:hover {
-	color:#0f597f;
-	text-decoration:none;
-	font-weight:normal;
-	outline:none;
-}
-b a:link, b a:visited {
-    font-weight: bold;
-}
+    font-weight: normal;
+    font-family: UbuntuBeta, Ubuntu, 'Bitstream Vera Sans', 'DejaVu Sans', Tahoma, sans-serif;
+    color: #000;
+    background: #fff;
+    }
+a {
+    color: #03A;
+    text-decoration: none;
+    }
+a:hover {
+    text-decoration: underline;
+    }
 img {
-	padding:0px;
-	margin:0px;
-	border:0px;
-	outline:none;
-}
-form {
-  margin: 0;
-  padding: 0;
-}
-tr td {
-  vertical-align: top;
-}
+    border: 0;
+    outline: none;
+    }
 .clear {
-	clear:both;
-	font-size:0px;
-	height:0px;
-}
-.fr {float:right;}
-.fl {float:left;}
-
+    clear: both;
+    font-size: 0;
+    height: 0;
+    }
+.fr {
+    float: right;
+    }
+.fl {
+    float: left;
+    }
 ul {
-    list-style:none;
+    list-style: none;
+    }
+table {
+    caption-side: top;
+    width: 100%;
+    border: none;
+    border-spacing: 0;
+    border-collapse: collapse;
+    }
+tr td {
+    vertical-align: top;
+    }
+th, td {
+    border-color: #D2D2D2;
+    }
+th {
+    text-align: left;
+    color: #5A5A5A;
+    padding: 4px 8px;
+    border-width: 1px;
+    border-style: solid;
+    }
+td {
+    padding: 4px 4px;
+    border-width: 0 0 1px 0;
+    border-style: dotted;
+    }
+h1 {
+    font-size:138.5%;
+    padding: 10px 0 5px 0;
+    }
+h2 {
+    font-size: 123.1%;
+    padding: 10px 0 5px 0;
+    }
+p {
+    padding: 5px 0;
+    }
+p img {
+    vertical-align: middle;
+    }
+
+
+/* =========================
+   Header & Navigation
+*/
+
+.black-link {
+    border-bottom: 1px solid #D2D2D2;
+    padding: 6px 1% 8px 1%;
+    font-size: 85%;
 }
 
-/* Loggerhead Menu tabs */
+/* Branch Title */
+h1.branch-name {
+    font-size: 131%;
+    padding: 0;
+    margin: 20px 0 0 1%;
+    }
+
+/* Loggerhead Menu Tabs */
 ul#menuTabs {
-	width:98%;
-	height:33px;
-	list-style:none;
-	color:#fff;
-	background:#fff url(../images/bg_menuTabs.gif) bottom repeat-x;
-	margin:0px;
-	padding:12px 0px 0px 2%;
-}
-ul#menuTabs li a:link, ul#menuTabs li a:visited {
-	float:left;
-	width:93px;
-	padding:10px 0px 0px 0px;
-	height:23px;
-	font:normal 14px/1.5 sans-serif;
-	text-align:center;
-	color:#fff;
-	text-decoration:none;
-	background:#ad2001 url(../images/bg_Tabs.gif) bottom center;
-}
+    height: 33px;
+    color: #fff;
+    margin: 0;
+    padding: 15px 20px 0 1%;
+    }
 ul#menuTabs li {
-	float:left;
-	color:#fff;
-	background:#f00 url(../images/bg_Tabs.gif) bottom center;
-}
+    float: left;
+    padding: 0 7px 0 0;
+    }
+ul#menuTabs li a {
+    padding: 3px 6px;
+    color: #000;
+    }
 ul#menuTabs li a:hover {
-	float:left;
-	color:#000;
-	background:#666 url(../images/bg_Tabs.gif) top center;
-	text-decoration:none;
-}
-ul#menuTabs li a#on:link, ul#menuTabs li a#on:visited, ul#menuTabs a#on:hover {
-	float:left;
-	color:#000;
-	background:#666 url(../images/bg_Tabs.gif) top center;
-	text-decoration:none;
-}
-/*Sub menu tabs*/
+    background-color: #f3f3f3;
+    text-decoration: none;
+    }
+ul#menuTabs li a#on {
+    font-weight: bold;
+    color: #fff;
+    background-color: #747474;
+    }
+
+/* Sub-menu Tabs */
 ul#submenuTabs {
-	width:100%;
-	height:24px;
-	list-style:none;
-	color:#fff;
-	background:#fff;
-	margin:0px;
-	padding:10px 0%;
-}
-ul#submenuTabs li a:link, ul#submenuTabs li a:visited {
-	float:left;
-	display:inline;
-	height:22px;
-	padding:2px 15px 0px 15px;
-	font:normal 14px/1.5 sans-serif;
-	text-align:center;
-	color:#fff;
-	text-decoration:none;
-	background:#d18b39;
-}
+    height: 24px;
+    margin-bottom: 10px;
+    }
 ul#submenuTabs li {
-	float:left;
-	display:inline;
-	margin-left:1px;
-	color:#fff;
-	background:#d18b39;
-}
-ul#submenuTabs li a:hover {
-	float:left;
-	display:inline;
-	color:#fff;
-	background:#d18b39;
-	text-decoration:underline;
-}
-ul#submenuTabs li#first a, ul#submenuTabs li#first a:link, ul#submenuTabs li#first a:visited, ul#submenuTabs li#first a:hover {
-	background:#d18b39 url(../images/bg_submenuTabs.gif) top left no-repeat;
-}
-ul#submenuTabs li#last a, ul#submenuTabs li#last a:link, ul#submenuTabs li#last a:visited, ul#submenuTabs li#last a:hover {
-	background:#d18b39 url(../images/bg_submenuTabs.gif) bottom right no-repeat;
-}
-
-/* Search & RSS */
-#finderBox {
-	width:300px;
-	float:right;
-	padding: 10px 0px 0px 0px;
-}
-#finderBox label {
-	float:left;
-	display:inline;
-	width:100px;
-	font:normal 12px/normal sans-serif;
-	text-align:right;
-	color:#000;
-	padding:5px 5px 0px 0px;
-}
-#finderBox input {
-	float:left;
-	font:normal 12px/normal sans-serif;
-	color:#666;
-	display:inline;
-	width:130px;
-	padding:3px;
-	background:#fff url(../images/bg_search_input.gif) 110px 3px no-repeat;
-	border:1px solid #e9e9e9;
-}
-#search_terms {
-    display:none;
-    border:solid 1px black;
-    width:180px;
-    overflow:auto;
-    background: #fff;
-}
-.rssfeed {
-	float:left;
-	display:inline;
-	padding:0px 0px 0px 10px;
-}
-
-/* General Content */
+    float: left;
+    display: inline;
+    margin-right: 20px;
+    }
+ul#submenuTabs li a {
+    float: left;
+    display: inline;
+    height: 22px;
+    padding: 0 0 0 18px;
+    background: transparent url(../images/ico_description.gif) no-repeat center left;
+    }
+
+/* Breadcrumbs */
+#breadcrumbs {
+    font-size: 77%;
+    margin-bottom: 10px;
+    }
+
+#branch-info {
+    width: 900px;
+    padding: 10px;
+    margin: 0 0 10px 0;
+    color: #666;
+    background: #fcfcfc;
+    border: 1px solid #b7b7b7;
+    }
+
+
+/* =========================
+   General Content
+*/
 #loggerheadCont {
-	width:98%;
-	padding:10px 1%;
-	color:#000;
-	background:#fff;
-}
-#loggerheadCont h1 {
-	font-size:18px;
-	color:#666;
-	margin:0px;
-	padding:10px 0px 5px 0px;
-}
-#loggerheadCont h1 span {
-    color:#889ea9;
-}
-#loggerheadCont p {
-	color:#000;
-	margin:0px;
-	padding:5px 0px;
-}
-#loggerheadCont p.expand, #loggerheadCont p.expand a, #loggerheadCont p.expand a:link, #loggerheadCont p.expand a:visited, #loggerheadCont p.expand a:hover {
-	color:#000;
-	text-decoration:none;
-}
-#loggerheadCont p img {
-	vertical-align:middle;
-}
-.link {
-	padding:0px 11px 0px 0px;
-	margin:0px 2px 0px 0px;
-	font-weight:bold;
-}
-.revnolink a, .revnolink a:visited, .revnolink a:link, .revnolink a:hover {
-	color:#CC6600;
-	text-decoration:none;
-}
-
-/*Log Table*/
+    width: 98%;
+    padding: 0 1% 10px 1%;
+    }
+
+
+/* =========================
+   Log Table
+*/
 #logentries {
-	caption-side:top;
-	width:99%;
-	margin:0px auto 10px auto;
-	padding:0px;
-	background:#fff;
-	border:none;
-	border-spacing:0px;
-	border-collapse:collapse;
-}
-.logheader {
-	background:#fff;
-	font:bold 14px/normal sans-serif;
-	padding:0px;
-	margin:0px;
-}
-.blueRow0, .blueRow1 {
-	background:#eef6fa;
-	font:normal 14px/normal sans-serif;
-	padding:0px;
-	margin:0px;
-}
-.blueRow1 {
-	background:#f8fbfd;
-}
-.revisionnumber, .expandcell, .diffcell, .downloadcell, .revnro, .expcell, .diffr, .downr, .allicones {
-	padding:4px 0px;
-	width:42px;
-	text-align:center;
-	border:1px solid #d2d2d2;
-}
-.expandcell, .expcell {
-	width:28px;
-}
-.summarycell, .authorcell, .tagscell, .datecell, .timedate{
-  padding: 4px 8px;
-  border: 1px solid #d2d2d2;
-}
-.icocell {
-  width: 18px;
-}
-.authorcell, .autcell {
-  width: 240px;
-}
-.tagscell, .tagcell {
-  width: 240px;
-}
-.datecell, .date {
-  width: 180px;
-}
-.timedate, .timedate2 {
-	width:80px;
-}
-.downloadcell, .downr {
-	width:47px;
-}
-.revnro, .expcell, .diffr, .summcell, .autcell, .tagcell, .icocell, .date, .downr, .allicones, .timedate2 {
-  border: 0;
-  padding: 4px 4px;
-  border-bottom:1px solid #d2d2d2;
-}
+    margin-bottom: 10px;
+    clear: left;
+    }
+p.expand, p.codin {
+    margin-right: 10px;
+    float: left;
+    }
+p.expand a, p.codin a {
+    color: #000;
+    text-decoration: none;
+    }
+.show_if_js {
+    display: none;
+    }
+
+/* th and td styles */
+th.revisionnumber,
+th.expandcell,
+th.diffcell,
+th.downloadcell {
+    padding-left: 0;
+    padding-right: 0;
+    text-align: center;
+    }
+th.diffcell,
+th.revisionnumber {
+    width: 42px;
+    }
+th.expandcell,
+td.expcell {
+    width: 10px;
+    }
+td.icocell {
+    width: 18px;
+    }
+th.authorcell,
+td.autcell,
+th.tagscell,
+td.tagcell {
+    width: 240px;
+    }
+th.datecell,
+td.date {
+    width: 180px;
+    }
+th.timedate,
+td.timedate2 {
+    width: 80px;
+    }
+th.downloadcell,
+td.downr {
+    width: 47px;
+    }
+
+
+/* =========================
+   Revision List
+*/
+.revision_log {
+    overflow: hidden;
+    }
 .revisioninfo {
-    margin-top:4px;
-}
+    margin-top: 4px;
+    }
 ul.expandrev {
-	list-style:none;
-	font:normal 11px/normal sans-serif;
-	color:#000;
-}
+    font-size: 85%;
+    }
 ul.expandrev li {
-	padding:2px 0px 2px 20px;
-	margin:0px 0px 2px 0px;
-}
+    padding: 2px 0 2px 20px;
+    margin: 0 0 2px 0;
+    }
 ul.expandrev li.mfrom {
-	background:url(../images/ico_mergefrom.gif) 0px 0px no-repeat;
-}
+    background: url(../images/ico_mergefrom.gif) 0 0 no-repeat;
+    }
 ul.expandrev li.mto {
-	background:url(../images/ico_mergeto.gif) 0px 0px no-repeat;
-}
+    background: url(../images/ico_mergeto.gif) 0 0 no-repeat;
+    }
 ul.expandrev li.committerli {
-	background:url(../images/ico_committer.gif) 2px 0px no-repeat;
-}
+    background: url(../images/ico_committer.gif) 2px 0 no-repeat;
+    }
 ul.expandrev li.desc {
-	background:url(../images/ico_description.gif) 0px 0px no-repeat;
-	color:#666;
-}
+    background: url(../images/ico_description.gif) 0 0 no-repeat;
+    color: #666;
+    }
 ul.expandrev li.files {
-	background:url(../images/ico_file.gif) 2px 0px no-repeat;
+    background: url(../images/ico_file.gif) 2px 0 no-repeat;
     margin-left: 8px;
-}
-
-/* Navigation */
-ul {border:0; margin:0; padding:0;}
-
+    }
+.revnolink a {
+    color: #000;
+    }
+
+
+/* =========================
+   Pagination
+*/
 ul#pages {
-	float:right;
-}
+    float: right;
+    }
 #pages li {
-	border:0; margin:0; padding:0;
-	font-size:12px;
-	list-style:none;
-	float:left;
-	display:inline;
-}
-#pages a {
-	border:solid 1px #DDDDDD;
-	margin-right:2px;
-}
-#pages .next a, #pages .previous a {
-	font-weight:bold;
-	border:solid 1px #fff;
-}
-#pages .active {
-	color:#f37b06;
-	font-weight:bold;
-	display:block;
-	float:left;
-	padding:4px 6px;
-}
-#pages a:link, #pages a:visited {
-	color:#f37b06;
-	display:block;
-	float:left;
-	padding:3px 6px;
-	text-decoration:none;
-}
-#pages .next a {
-	color:#0f6b97;
-}
-#pages a:hover {
-	border:solid 1px #666666;
-}
-.breadcrumb a {
-	color:#889ea9;
-	font-weight:bold;
-	font-size:18px;
-}
-.breadcrumb a:hover{
-	color:#666;
-	font-weight:bold;
-	font-size:18px;
-}
-#branch-info {
-	width:900px;
-	padding:10px 10px 10px 10px;
-	margin:0px 0px 10px 0px;
-	color:#666;
-	background:#fcfcfc;
-	border:1px solid #b7b7b7;
-}
-.show_if_js {
+    font-size: 77%;
+    font-weight: bold;
+    float: left;
+    display: inline;
+    padding: 3px 6px;
+    margin-right: 2px;
+    }
+
+
+/* =========================
+   Search & RSS
+*/
+#finderBox {
+    width: 300px;
+    float: right;
+    padding: 10px 0 0 0;
+    }
+#finderBox label,
+#finderBox input {
+    float: left;
+    font-weight: normal;
+    font-size: 12px;
+    }
+#finderBox label {
+    display: inline;
+    width: 100px;
+    text-align: right;
+    color: #000;
+    padding: 5px 5px 0 0;
+    }
+#finderBox input {
+    color: #666;
+    display: inline;
+    width: 130px;
+    padding: 3px;
+    background: #fff url(../images/bg_search_input.gif) 110px 3px no-repeat;
+    border: 1px solid #e9e9e9;
+    }
+#search_terms {
     display: none;
-}
-.revision_log {
-    overflow: hidden;
-}
+    border: solid 1px black;
+    width: 180px;
+    overflow: auto;
+    background: #fff;
+    }
+.rssfeed {
+    float: left;
+    display: inline;
+    padding: 0 0 0 10px;
+    }
+
+
+/* =========================
+   Footer
+*/
+#footer {
+    font-size: 77%;
+    }

=== modified file 'loggerhead/static/css/view.css'
--- loggerhead/static/css/view.css	2010-12-01 08:30:02 +0000
+++ loggerhead/static/css/view.css	2011-02-18 05:41:17 +0000
@@ -1,31 +1,43 @@
 /*table*/
-.viewLineTit, .viewLine, .viewRevTit, .viewRev,
-.viewContTit, .viewCont
-{
-	border:1px solid #d2d2d2;
-}
-.viewLine, .viewRev, .viewComm, .viewCont {
-	border:none;
-}
-.viewLine, .viewLineTit {
-	padding-right: .2em;
-}
-.viewLine {
-	width:37px;
-}
-.viewContTit, .viewCont {
-	width:auto;
-	padding-left: .3em;
-}
-.viewRevTit, .viewRev {
-	width:70px;
-	text-align:center;
-}
-.viewLine, .viewCont {
-	font:normal 12px/normal monospace;
-        whitespace: pre;
-}
-.viewCont pre { margin: 0; }
-.viewLine {
-	text-align:right;
-}
+.viewLineTit,
+.viewRevTit,
+.viewContTit {
+    border: 1px solid #d2d2d2;
+    }
+.viewLine,
+.viewRev,
+.viewComm,
+.viewCont {
+    border: none;
+    }
+.viewLine,
+.viewLineTit {
+    padding-right: .2em;
+    }
+.viewLine {
+    width: 37px;
+    }
+.viewContTit,
+.viewCont {
+    width: auto;
+    padding-left: .3em;
+    }
+.viewRevTit,
+.viewRev {
+    width: 70px;
+    text-align: center;
+    }
+.viewLine,
+.viewCont {
+    font-size: 116%;
+    font-family: 'UbuntuBeta Mono', 'Ubuntu Mono', monospace;
+    line-height: 108%;
+    whitespace: pre;
+    }
+.viewCont pre {
+    margin: 0;
+    }
+.viewLine {
+    text-align: right;
+    font-size: 93%;
+    }

=== added directory 'loggerhead/static/downloads'
=== added file 'loggerhead/static/images/favicon.png'
Binary files loggerhead/static/images/favicon.png and loggerhead/static/images/favicon.png differ
=== modified file 'loggerhead/templatefunctions.py'
--- loggerhead/templatefunctions.py	2010-12-01 08:30:02 +0000
+++ loggerhead/templatefunctions.py	2011-03-02 14:07:21 +0000
@@ -186,12 +186,8 @@
 
         # TODO: On old Python versions, elementtree may be used.
 
-        try:
-            import simplejson
-        except ImportError:
-            pass
-        else:
-            versions.append(('simplejson', simplejson.__version__))
+        import simplejson
+        versions.append(('simplejson', simplejson.__version__))
 
         try:
             Dozer = pkg_resources.get_distribution('Dozer')

=== modified file 'loggerhead/templates/browse.pt'
--- loggerhead/templates/browse.pt	2009-10-17 06:55:25 +0000
+++ loggerhead/templates/browse.pt	2011-02-17 19:51:25 +0000
@@ -4,6 +4,8 @@
     <title>loggerhead branches</title>
     <link rel="stylesheet"
           tal:attributes="href python:branch.static_url('/static/css/global.css')" />
+    <link rel="shortcut icon"
+          tal:attributes="href python:branch.static_url('/static/images/favicon.ico')" />
   </head>
 
   <body>

=== modified file 'loggerhead/templates/changelog.pt'
--- loggerhead/templates/changelog.pt	2009-07-21 21:52:59 +0000
+++ loggerhead/templates/changelog.pt	2011-02-23 03:01:15 +0000
@@ -16,26 +16,36 @@
   </head>
 
   <body>
+      <tal:block metal:fill-slot="backlink">
+          <tal:has-link condition="branch/branch_link">
+            <div class="black-link">
+                <a tal:attributes="href branch/branch_link">
+                  &#8592; Back to branch summary
+                </a>
+            </div>
+          </tal:has-link>
+      </tal:block>
+    <tal:block metal:fill-slot="branchname" tal:content="string:${branch/friendly_name}" />
     <tal:block metal:fill-slot="heading">
-      <h1>
-        <tal:has-link condition="branch/branch_link">
-          <a tal:attributes="href branch/branch_link"
-             tal:content="branch/friendly_name">
-            nice/branch/name
-          </a>
-        </tal:has-link>
-        <tal:no-link condition="not: branch/branch_link">
-          <span metal:use-macro="breadcrumbs/directory"></span>
-        </tal:no-link>
-        <tal:block condition="changes">
-          : changes
-          <tal:block condition="filter_file_id">to <span tal:content="python:history.get_path(revid, filter_file_id)" /></tal:block>
-          <tal:block condition="start_revid">from revision
-            <span tal:content="python:history.get_revno(start_revid)"/>
-          </tal:block>
-          <tal:block condition="query">matching <tal:block content="query" /></tal:block>
-        </tal:block>
-      </h1>
+        <div id="breadcrumbs">
+            <tal:has-link condition="branch/branch_link">
+              <a tal:attributes="href branch/branch_link"
+                 tal:content="branch/friendly_name">
+                nice/branch/name
+              </a>
+            </tal:has-link>
+            <tal:no-link condition="not: branch/branch_link">
+              <span metal:use-macro="breadcrumbs/directory"></span>
+            </tal:no-link>
+            <tal:block condition="changes">
+              &#187; Changes
+              <tal:block condition="filter_file_id">to <span tal:content="python:history.get_path(revid, filter_file_id)" /></tal:block>
+              <tal:block condition="start_revid">from revision
+                <span tal:content="python:history.get_revno(start_revid)"/>
+              </tal:block>
+              <tal:block condition="query">matching <tal:block content="query" /></tal:block>
+            </tal:block>
+        </div>
     </tal:block>
 
     <div metal:fill-slot="content">
@@ -69,14 +79,14 @@
       <!-- Table -->
       <table id="logentries">
         <tr class="logheader">
-          <td class="revisionnumber">Rev</td>
-          <td class="expandcell show_if_js">&nbsp;</td>
-          <td class="summarycell">Summary</td>
-          <td class="authorcell">Authors</td>
-          <td tal:condition="show_tag_col" class="tagscell">Tags</td>
-          <td class="datecell">Date</td>
-          <td class="diffcell">Diff</td>
-          <td class="downloadcell">Files</td>
+          <th class="revisionnumber">Rev</th>
+          <th class="expandcell show_if_js">&nbsp;</th>
+          <th class="summarycell">Summary</th>
+          <th class="authorcell">Authors</th>
+          <th tal:condition="show_tag_col" class="tagscell">Tags</th>
+          <th class="datecell">Date</th>
+          <th class="diffcell">Diff</th>
+          <th class="downloadcell">Files</th>
         </tr>
         <tal:block tal:repeat="entry changes">
           <a tal:attributes="name string:entry-${entry/revno}"/>

=== modified file 'loggerhead/templates/directory.pt'
--- loggerhead/templates/directory.pt	2010-04-13 23:35:01 +0000
+++ loggerhead/templates/directory.pt	2011-02-17 19:51:25 +0000
@@ -4,6 +4,8 @@
     <title tal:content="string:Browsing ${name}"></title>
     <link rel="stylesheet"
           tal:attributes="href python:static_url('/static/css/global.css')" />
+    <link rel="shortcut icon"
+          tal:attributes="href python:branch.static_url('/static/images/favicon.ico')" /> 
   </head>
   <body>
     <div id="loggerheadCont">

=== modified file 'loggerhead/templates/inventory.pt'
--- loggerhead/templates/inventory.pt	2010-12-01 08:30:02 +0000
+++ loggerhead/templates/inventory.pt	2011-02-23 03:01:15 +0000
@@ -9,23 +9,34 @@
     </metal:block>
   </head>
   <body>
-
-    <h1 metal:fill-slot="heading">
-      <tal:has-link condition="branch/branch_link">
-        <a tal:attributes="href branch/branch_link"
-           tal:content="branch/friendly_name">
-          nice/branch/name
-        </a>
-      </tal:has-link>
-      <span tal:condition="not: branch/branch_link">
-        <span metal:use-macro="breadcrumbs/directory" />
-      </span>
-      <tal:block condition="python:change">
-        : viewing
-        <span metal:use-macro="breadcrumbs/branch" />
-        for revision <span tal:content="change/revno"/>
-      </tal:block>
-    </h1>
+      <tal:block metal:fill-slot="backlink">
+          <tal:has-link condition="branch/branch_link">
+            <div class="black-link">
+                <a tal:attributes="href branch/branch_link">
+                  &#8592; Back to branch summary
+                </a>
+            </div>
+          </tal:has-link>
+      </tal:block>
+      <tal:block metal:fill-slot="branchname" tal:content="string:${branch/friendly_name}" />
+      <tal:block metal:fill-slot="heading">
+      <div id="breadcrumbs">
+          <tal:has-link condition="branch/branch_link">
+            <a tal:attributes="href branch/branch_link"
+               tal:content="branch/friendly_name">
+              nice/branch/name
+            </a>
+          </tal:has-link>
+          <span tal:condition="not: branch/branch_link">
+            <span metal:use-macro="breadcrumbs/directory" />
+          </span>
+          <tal:block condition="python:change">
+            &#187; Viewing
+            <span metal:use-macro="breadcrumbs/branch" />
+            for revision <span tal:content="change/revno"/>
+          </tal:block>
+        </div>
+      </tal:block>
 
     <div metal:fill-slot="content">
         <tal:branch-info replace="structure python:branchinfo(branch)" />

=== modified file 'loggerhead/templates/macros.pt'
--- loggerhead/templates/macros.pt	2009-07-21 04:39:33 +0000
+++ loggerhead/templates/macros.pt	2011-03-03 10:58:57 +0000
@@ -9,6 +9,8 @@
     <title metal:define-slot="title"></title>
     <link rel="stylesheet"
           tal:attributes="href python:branch.static_url('/static/css/global.css')" />
+    <link rel="shortcut icon"
+          tal:attributes="href python:branch.static_url('/static/images/favicon.png')" />
     <tal:comment condition="nothing">
       <script type="text/javascript"
               src="http://getfirebug.com/releases/lite/1.2/firebug-lite-compressed.js"></script>
@@ -43,17 +45,21 @@
 
 <body>
     <!-- Loggerhead Content Area -->
-
+      
+      <div metal:define-slot="backlink" />
+      
+      <h1 class="branch-name">
+        <span metal:define-slot="branchname" />
+      </h1>
       <tal:menu define="fileview_active fileview_active"
                 replace="structure python:menu(branch, url, fileview_active)" />
       <div id="loggerheadCont">
 
         <div id="search_terms"></div>
-
-        <h1 metal:define-slot="heading"></h1>
+        <div metal:define-slot="heading" />
         <div metal:define-slot="content"></div>
 
-        <p class="fl">Loggerhead <tal:version content="loggerhead_version" /> is a web-based interface for <a href="http://bazaar-vcs.org/">Bazaar</a> branches</p>
+        <p id="footer" class="fl">Loggerhead <tal:version content="loggerhead_version" /> is a web-based interface for <a href="http://bazaar-vcs.org/">Bazaar</a> branches</p>
       </div>
   </body>
 </html>

=== modified file 'loggerhead/templates/menu.pt'
--- loggerhead/templates/menu.pt	2008-08-14 22:45:52 +0000
+++ loggerhead/templates/menu.pt	2011-02-09 22:33:59 +0000
@@ -13,8 +13,6 @@
         <li><a tal:attributes="href python:url('/files', clear=1);
                                title string:Files;
                                id string:on">Files</a></li>
-        <li><a tal:attributes="href string:https://help.launchpad.net/Loggerhead;
-                               title string:Help">Help</a></li> 
         </tal:fileview-active>
       </ul>
 </tal:menu>

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2010-12-01 08:30:02 +0000
+++ loggerhead/templates/revision.pt	2011-03-10 03:54:45 +0000
@@ -20,8 +20,18 @@
     </head>
 
     <body>
+      <tal:block metal:fill-slot="backlink">
+          <tal:has-link condition="branch/branch_link">
+            <div class="black-link">
+                <a tal:attributes="href branch/branch_link">
+                  &#8592; Back to branch summary
+                </a>
+            </div>
+          </tal:has-link>
+      </tal:block>
+      <tal:block metal:fill-slot="branchname" tal:content="string:${branch/friendly_name}" />
       <tal:block metal:fill-slot="heading">
-        <h1>
+        <div id="breadcrumbs">
           <tal:has-link condition="branch/branch_link">
             <a tal:attributes="href branch/branch_link"
                tal:content="branch/friendly_name">
@@ -31,7 +41,7 @@
           <tal:no-link condition="not: branch/branch_link">
             <span metal:use-macro="breadcrumbs/directory"></span>
           </tal:no-link>
-          <span class="breadcrumb">:
+          <span class="breadcrumb">&#187; Revision
             <tal:revno condition="not:specific_path" content="change/revno"></tal:revno>
             <a tal:condition="specific_path" tal:content="change/revno"
                tal:attributes="href python:url(['/revision', change.revno])"
@@ -43,11 +53,11 @@
           <span class="breadcrumb" tal:condition="specific_path">
             : <tal:annotate content="structure python:view_link(url, change.revno, specific_path)" />
           </span>
-        </h1>
+        </div>
         <tal:branch-info replace="structure python:branchinfo(branch)" />
-        <p tal:condition="not:specific_path">
+        <h1 tal:condition="not:specific_path">
           Viewing all changes in revision <tal:revno content="change/revno" />.
-        </p>
+        </h1>
         <p tal:condition="specific_path">
           <a tal:attributes="href python:url(['/revision', change.revno])">
             &#xAB; back to all changes in this revision
@@ -74,7 +84,10 @@
                tal:attributes="href python:url(['/diff', change.revno], clear=1)">download diff</a>
             <a tal:condition="python:compare_revid is not None"
                tal:attributes="href python:url(['/diff', change.revno, history.get_revno(compare_revid)], clear=1)">download diff</a>
-          </li>
+           </li>
+           <li tal:condition="python:can_export">
+            <a tal:attributes="href python:url(['/tarball', change.revno])">download tarball</a>
+           </li>
           <li id="last"><a tal:attributes="href python:url(['/changes', change.revno]);
                                            title string:view history from revision ${change/revno}"
                            tal:content="string:view history from revision ${change/revno}"></a></li>

=== modified file 'loggerhead/templates/revisioninfo.pt'
--- loggerhead/templates/revisioninfo.pt	2010-01-11 03:26:44 +0000
+++ loggerhead/templates/revisioninfo.pt	2011-02-18 05:41:17 +0000
@@ -1,23 +1,41 @@
 <div class="infoContainer">
     <div id="infTxt">
-        <div class="committer" tal:content="python:', '.join(util.hide_emails(change.authors))"></div>
-        <div class="timer" tal:content="python:util.date_time(change.date)"></div>
-        <div class="mfrom" tal:repeat="parent python:change.parents[1:]">
-            <a tal:attributes="title string:Merged From ${parent/revno};
-                               href python:url(['/revision', parent.revno], clear=1)"
-               tal:content="python:revno_with_nick(parent)"></a>
-        </div>
-        <div class="mto" tal:repeat="merge_point change/merge_points">
-            <a tal:attributes="title string:Merged In ${merge_point/revno};
-                               href python:url(['/revision', merge_point.revno], clear=1)"
-               tal:content="python:revno_with_nick(merge_point)"></a>
-        </div>
-        <div class="mto" tal:condition="merged_in">
-          This revision was merged to the branch mainline in
-          <a tal:attributes="href python:url(['/revision', merged_in], clear=1)">revision
-            <tal:b content="merged_in"/></a>.
-        </div>
-        <div class="revid" tal:content="string:Revision ID: ${change/revid}"></div>
+        <ul>
+            <li class="committer">
+                <strong>Committer:</strong>
+                <span tal:content="python:', '.join(util.hide_emails(change.authors))"></span>
+            </li>
+            <li class="timer">
+                <strong>Date:</strong>
+                <span tal:content="python:util.date_time(change.date)"></span>
+            </li>
+            <li class="mfrom" tal:condition="python:change.parents[1:]">
+                <strong>mfrom:</strong>
+                <span tal:repeat="parent python:change.parents[1:]">
+                    <a tal:attributes="title string:Merged From ${parent/revno};
+                                       href python:url(['/revision', parent.revno], clear=1)"
+                       tal:content="python:revno_with_nick(parent)"></a>
+                </span>
+            </li>
+            <li class="mto" tal:condition="change/merge_points">
+                <strong>mto:</strong>
+                <span tal:repeat="merge_point change/merge_points">
+                    <a tal:attributes="title string:Merged In ${merge_point/revno};
+                                       href python:url(['/revision', merge_point.revno], clear=1)"
+                       tal:content="python:revno_with_nick(merge_point)"></a>
+                </span>
+            </li>
+            <li class="mto" tal:condition="merged_in">
+                <strong>mto:</strong>
+                  This revision was merged to the branch mainline in
+                  <a tal:attributes="href python:url(['/revision', merged_in], clear=1)">revision
+                    <tal:b content="merged_in"/></a>.
+            </li>
+            <li class="revid">
+                <strong>Revision ID:</strong>
+                <span tal:content="change/revid"></span>
+            </li>
+        </ul>
 	<div tal:attributes="class string:foreign-${change/foreign_vcs}"
 	     tal:condition="change/foreign_vcs" 
 	     tal:repeat="revid_part python:change.foreign_revid.keys()"

=== modified file 'loggerhead/templates/view.pt'
--- loggerhead/templates/view.pt	2010-12-01 08:30:02 +0000
+++ loggerhead/templates/view.pt	2011-02-23 03:01:15 +0000
@@ -15,19 +15,30 @@
   </head>
 
   <body>
-
-    <h1 metal:fill-slot="heading">
-      <tal:has-link condition="branch/branch_link">
-        <a tal:attributes="href branch/branch_link"
-           tal:content="branch/friendly_name">
-          nice/branch/name
-        </a>
-      </tal:has-link>
-      <tal:no-link condition="not: branch/branch_link">
-        <span metal:use-macro="breadcrumbs/directory"></span>
-      </tal:no-link>
-      <span>: <span metal:use-macro="breadcrumbs/branch" /> (revision <tal:revno content="change/revno"></tal:revno>)</span>
-    </h1>
+      <tal:block metal:fill-slot="backlink">
+          <tal:has-link condition="branch/branch_link">
+            <div class="black-link">
+                <a tal:attributes="href branch/branch_link">
+                  &#8592; Back to branch summary
+                </a>
+            </div>
+          </tal:has-link>
+      </tal:block>
+      <tal:block metal:fill-slot="branchname" tal:content="string:${branch/friendly_name}" />
+          <tal:block metal:fill-slot="heading">
+          <div id="breadcrumbs">
+          <tal:has-link condition="branch/branch_link">
+            <a tal:attributes="href branch/branch_link"
+               tal:content="branch/friendly_name">
+              nice/branch/name
+            </a>
+          </tal:has-link>
+          <tal:no-link condition="not: branch/branch_link">
+            <span metal:use-macro="breadcrumbs/directory"></span>
+          </tal:no-link>
+          <span>: <span metal:use-macro="breadcrumbs/branch" /> (revision <tal:revno content="change/revno"></tal:revno>)</span>
+        </div>
+      </tal:block>
 
     <div metal:fill-slot="content">
       <tal:branch-info replace="structure python:branchinfo(branch)" />
@@ -61,9 +72,9 @@
       <div class="view">
         <table id="logentries">
           <tr class="logheader">
-            <td class="viewLineTit">Line</td>
-            <td class="viewRevTit" tal:condition="annotated">Revision</td>
-            <td class="viewContTit">Contents</td>
+            <th class="viewLineTit">Line</th>
+            <th class="viewRevTit" tal:condition="annotated">Revision</th>
+            <th class="viewContTit">Contents</th>
           </tr>
 
           <tal:rep tal:repeat="line contents">

=== modified file 'loggerhead/tests/__init__.py'
--- loggerhead/tests/__init__.py	2010-05-10 19:36:37 +0000
+++ loggerhead/tests/__init__.py	2011-02-10 23:06:59 +0000
@@ -1,4 +1,4 @@
-# Copyright 2006, 2010 Canonical Ltd
+# Copyright 2006, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
         (__name__ + '.' + x) for x in [
             'test_controllers',
             'test_corners',
+            'test_load_test',
             'test_simple',
             'test_templating',
         ]]))

=== modified file 'loggerhead/tests/test_controllers.py'
--- loggerhead/tests/test_controllers.py	2009-03-19 18:48:15 +0000
+++ loggerhead/tests/test_controllers.py	2011-02-10 17:01:10 +0000
@@ -1,3 +1,10 @@
+from cStringIO import StringIO
+import logging
+
+from paste.httpexceptions import HTTPServerError
+
+from bzrlib import errors
+
 from loggerhead.apps.branch import BranchWSGIApp
 from loggerhead.controllers.annotate_ui import AnnotateUI
 from loggerhead.controllers.inventory_ui import InventoryUI
@@ -15,15 +22,50 @@
         tree.commit('')
         tree.branch.lock_read()
         self.addCleanup(tree.branch.unlock)
-        branch_app = BranchWSGIApp(tree.branch)
+        branch_app = BranchWSGIApp(tree.branch, '')
+        branch_app.log.setLevel(logging.CRITICAL)
+        # These are usually set in BranchWSGIApp.app(), which is set from env
+        # settings set by BranchesFromTransportRoot, so we fake it.
+        branch_app._static_url_base = '/'
+        branch_app._url_base = '/'
         return tree.branch, InventoryUI(branch_app, branch_app.get_history)
 
+    def consume_app(self, app, extra_environ=None):
+        env = {'SCRIPT_NAME': '/files', 'PATH_INFO': ''}
+        if extra_environ is not None:
+            env.update(extra_environ)
+        body = StringIO()
+        start = []
+        def start_response(status, headers, exc_info=None):
+            start.append((status, headers, exc_info))
+            return body.write
+        extra_content = list(app(env, start_response))
+        body.writelines(extra_content)
+        return start[0], body.getvalue()
+
     def test_get_filelist(self):
         bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
             ['filename'])
         inv = bzrbranch.repository.get_inventory(bzrbranch.last_revision())
         self.assertEqual(1, len(inv_ui.get_filelist(inv, '', 'filename')))
 
+    def test_smoke(self):
+        bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
+            ['filename'])
+        start, content = self.consume_app(inv_ui)
+        self.assertEqual(('200 OK', [('Content-Type', 'text/html')], None),
+                         start)
+        self.assertContainsRe(content, 'filename')
+
+    def test_no_content_for_HEAD(self):
+        bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
+            ['filename'])
+        start, content = self.consume_app(inv_ui,
+                            extra_environ={'REQUEST_METHOD': 'HEAD'})
+        self.assertEqual(('200 OK', [('Content-Type', 'text/html')], None),
+                         start)
+        self.assertEqual('', content)
+
 
 class TestRevisionUI(BasicTests):
 
@@ -61,20 +103,25 @@
 
     def make_annotate_ui_for_file_history(self, file_id, rev_ids_texts):
         tree = self.make_branch_and_tree('.')
-        open('filename', 'w').write('')
+        self.build_tree_contents([('filename', '')])
         tree.add(['filename'], [file_id])
         for rev_id, text in rev_ids_texts:
-            open('filename', 'w').write(text)
+            self.build_tree_contents([('filename', text)])
             tree.commit(rev_id=rev_id, message='.')
         tree.branch.lock_read()
         self.addCleanup(tree.branch.unlock)
-        branch_app = BranchWSGIApp(tree.branch)
+        branch_app = BranchWSGIApp(tree.branch, friendly_name='test_name')
         return AnnotateUI(branch_app, branch_app.get_history)
 
     def test_annotate_file(self):
         history = [('rev1', 'old\nold\n'), ('rev2', 'new\nold\n')]
         ann_ui = self.make_annotate_ui_for_file_history('file_id', history)
-        annotated = list(ann_ui.annotate_file('file_id', 'rev2'))
+        # A lot of this state is set up by __call__, but we'll do it directly
+        # here.
+        ann_ui.args = ['rev2']
+        annotate_info = ann_ui.get_values('filename',
+            kwargs={'file_id': 'file_id'}, headers={})
+        annotated = list(annotate_info['annotated'])
         self.assertEqual(2, len(annotated))
         self.assertEqual('2', annotated[0].change.revno)
         self.assertEqual('1', annotated[1].change.revno)

=== added file 'loggerhead/tests/test_load_test.py'
--- loggerhead/tests/test_load_test.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_load_test.py	2011-02-10 00:49:06 +0000
@@ -0,0 +1,337 @@
+# Copyright 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#
+
+"""Tests for the load testing code."""
+
+import socket
+import time
+import threading
+import Queue
+
+from bzrlib import tests
+from bzrlib.tests import http_server
+
+from bzrlib.plugins.loggerhead.loggerhead import load_test
+
+
+empty_script = """{
+    "parameters": {},
+    "requests": []
+}"""
+
+class TestRequestDescription(tests.TestCase):
+
+    def test_init_from_dict(self):
+        rd = load_test.RequestDescription({'thread': '10', 'relpath': '/foo'})
+        self.assertEqual('10', rd.thread)
+        self.assertEqual('/foo', rd.relpath)
+
+    def test_default_thread_is_1(self):
+        rd = load_test.RequestDescription({'relpath': '/bar'})
+        self.assertEqual('1', rd.thread)
+        self.assertEqual('/bar', rd.relpath)
+
+
+_cur_time = time.time()
+def one_sec_timer():
+    """Every time this timer is called, it increments by 1 second."""
+    global _cur_time
+    _cur_time += 1.0
+    return _cur_time
+
+
+class NoopRequestWorker(load_test.RequestWorker):
+
+    # Every call to _timer will increment by one
+    _timer = staticmethod(one_sec_timer)
+
+    # Ensure that process never does anything
+    def process(self, url):
+        return True
+
+
+class TestRequestWorkerInfrastructure(tests.TestCase):
+    """Tests various infrastructure bits, without doing actual requests."""
+
+    def test_step_next_tracks_time(self):
+        rt = NoopRequestWorker('id')
+        rt.queue.put('item')
+        rt.step_next()
+        self.assertTrue(rt.queue.empty())
+        self.assertEqual([('item', True, 1.0)], rt.stats)
+
+    def test_step_multiple_items(self):
+        rt = NoopRequestWorker('id')
+        rt.queue.put('item')
+        rt.step_next()
+        rt.queue.put('next-item')
+        rt.step_next()
+        self.assertTrue(rt.queue.empty())
+        self.assertEqual([('item', True, 1.0), ('next-item', True, 1.0)],
+                         rt.stats)
+
+    def test_step_next_does_nothing_for_noop(self):
+        rt = NoopRequestWorker('id')
+        rt.queue.put('item')
+        rt.step_next()
+        rt.queue.put('<noop>')
+        rt.step_next()
+        self.assertEqual([('item', True, 1.0)], rt.stats)
+
+    def test_step_next_will_timeout(self):
+        # We don't want step_next to block forever
+        rt = NoopRequestWorker('id', blocking_time=0.001)
+        self.assertRaises(Queue.Empty, rt.step_next)
+
+    def test_run_stops_for_stop_event(self):
+        rt = NoopRequestWorker('id', blocking_time=0.001, _queue_size=2)
+        rt.queue.put('item1')
+        rt.queue.put('item2')
+        event = threading.Event()
+        t = threading.Thread(target=rt.run, args=(event,))
+        t.start()
+        # Wait for the queue to be processed
+        rt.queue.join()
+        # Now we can queue up another one, and wait for it
+        rt.queue.put('item3')
+        rt.queue.join()
+        # Now set the stopping event
+        event.set()
+        # Add another item to the queue, which might get processed, but the
+        # next item won't
+        rt.queue.put('item4')
+        rt.queue.put('item5')
+        t.join()
+        self.assertEqual([('item1', True, 1.0), ('item2', True, 1.0),
+                          ('item3', True, 1.0)],
+                         rt.stats[:3])
+        # The last event might be item4 or might be item3, the important thing
+        # is that even though there are still queued events, we won't
+        # process anything past the first
+        self.assertNotEqual('item5', rt.stats[-1][0])
+
+
+class TestRequestWorker(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestRequestWorker, self).setUp()
+        self.transport_readonly_server = http_server.HttpServer
+
+    def test_request_items(self):
+        rt = load_test.RequestWorker('id', blocking_time=0.01, _queue_size=2)
+        self.build_tree(['file1', 'file2'])
+        readonly_url1 = self.get_readonly_url('file1')
+        self.assertStartsWith(readonly_url1, 'http://')
+        readonly_url2 = self.get_readonly_url('file2')
+        rt.queue.put(readonly_url1)
+        rt.queue.put(readonly_url2)
+        rt.step_next()
+        rt.step_next()
+        self.assertEqual(readonly_url1, rt.stats[0][0])
+        self.assertEqual(readonly_url2, rt.stats[1][0])
+
+    def test_request_nonexistant_items(self):
+        rt = load_test.RequestWorker('id', blocking_time=0.01, _queue_size=2)
+        readonly_url1 = self.get_readonly_url('no-file1')
+        rt.queue.put(readonly_url1)
+        rt.step_next()
+        self.assertEqual(readonly_url1, rt.stats[0][0])
+        self.assertEqual(False, rt.stats[0][1])
+
+    def test_no_server(self):
+        s = socket.socket()
+        # Bind to a port, but don't listen on it
+        s.bind(('localhost', 0))
+        url = 'http://%s:%s/path' % s.getsockname()
+        rt = load_test.RequestWorker('id', blocking_time=0.01, _queue_size=2)
+        rt.queue.put(url)
+        rt.step_next()
+        self.assertEqual((url, False), rt.stats[0][:2])
+
+
+class NoActionScript(load_test.ActionScript):
+
+    _thread_class = NoopRequestWorker
+    _default_blocking_timeout = 0.01
+
+
+class TestActionScriptInfrastructure(tests.TestCase):
+
+    def test_parse_requires_parameters_and_requests(self):
+        self.assertRaises(ValueError,
+            load_test.ActionScript.parse, '')
+        self.assertRaises(ValueError,
+            load_test.ActionScript.parse, '{}')
+        self.assertRaises(ValueError,
+            load_test.ActionScript.parse, '{"parameters": {}}')
+        self.assertRaises(ValueError,
+            load_test.ActionScript.parse, '{"requests": []}')
+        load_test.ActionScript.parse(
+            '{"parameters": {}, "requests": [], "comment": "section"}')
+        script = load_test.ActionScript.parse(
+            empty_script)
+        self.assertIsNot(None, script)
+
+    def test_parse_default_base_url(self):
+        script = load_test.ActionScript.parse(empty_script)
+        self.assertEqual('http://localhost:8080', script.base_url)
+
+    def test_parse_find_base_url(self):
+        script = load_test.ActionScript.parse(
+            '{"parameters": {"base_url": "http://example.com"},'
+            ' "requests": []}')
+        self.assertEqual('http://example.com', script.base_url)
+
+    def test_parse_default_blocking_timeout(self):
+        script = load_test.ActionScript.parse(empty_script)
+        self.assertEqual(60.0, script.blocking_timeout)
+
+    def test_parse_find_blocking_timeout(self):
+        script = load_test.ActionScript.parse(
+            '{"parameters": {"blocking_timeout": 10.0},'
+            ' "requests": []}'
+        )
+        self.assertEqual(10.0, script.blocking_timeout)
+
+    def test_parse_finds_requests(self):
+        script = load_test.ActionScript.parse(
+            '{"parameters": {}, "requests": ['
+            ' {"relpath": "/foo"},'
+            ' {"relpath": "/bar"}'
+            ' ]}')
+        self.assertEqual(2, len(script._requests))
+        self.assertEqual("/foo", script._requests[0].relpath)
+        self.assertEqual("/bar", script._requests[1].relpath)
+
+    def test__get_worker(self):
+        script = NoActionScript()
+        # If an id is found, then we should create it
+        self.assertEqual({}, script._threads)
+        worker = script._get_worker('id')
+        self.assertTrue('id' in script._threads)
+        # We should have set the blocking timeout
+        self.assertEqual(script.blocking_timeout / 10.0,
+                         worker.blocking_time)
+
+        # Another request will return the identical object
+        self.assertIs(worker, script._get_worker('id'))
+
+        # And the stop event will stop the thread
+        script.stop_and_join()
+
+    def test__full_url(self):
+        script = NoActionScript()
+        self.assertEqual('http://localhost:8080/path',
+                         script._full_url('/path'))
+        self.assertEqual('http://localhost:8080/path/to/foo',
+                         script._full_url('/path/to/foo'))
+        script.base_url = 'http://example.com'
+        self.assertEqual('http://example.com/path/to/foo',
+                         script._full_url('/path/to/foo'))
+        script.base_url = 'http://example.com/base'
+        self.assertEqual('http://example.com/base/path/to/foo',
+                         script._full_url('/path/to/foo'))
+        script.base_url = 'http://example.com'
+        self.assertEqual('http://example.com:8080/path',
+                         script._full_url(':8080/path'))
+
+    def test_single_threaded(self):
+        script = NoActionScript.parse("""{
+            "parameters": {"base_url": ""},
+            "requests": [
+                {"thread": "1", "relpath": "first"},
+                {"thread": "1", "relpath": "second"},
+                {"thread": "1", "relpath": "third"},
+                {"thread": "1", "relpath": "fourth"}
+            ]}""")
+        script.run()
+        worker = script._get_worker("1")
+        self.assertEqual(["first", "second", "third", "fourth"],
+                         [s[0] for s in worker.stats])
+
+    def test_two_threads(self):
+        script = NoActionScript.parse("""{
+            "parameters": {"base_url": ""},
+            "requests": [
+                {"thread": "1", "relpath": "first"},
+                {"thread": "2", "relpath": "second"},
+                {"thread": "1", "relpath": "third"},
+                {"thread": "2", "relpath": "fourth"}
+            ]}""")
+        script.run()
+        worker = script._get_worker("1")
+        self.assertEqual(["first", "third"],
+                         [s[0] for s in worker.stats])
+        worker = script._get_worker("2")
+        self.assertEqual(["second", "fourth"],
+                         [s[0] for s in worker.stats])
+
+
+class TestActionScriptIntegration(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestActionScriptIntegration, self).setUp()
+        self.transport_readonly_server = http_server.HttpServer
+
+    def test_full_integration(self):
+        self.build_tree(['first', 'second', 'third', 'fourth'])
+        url = self.get_readonly_url()
+        script = load_test.ActionScript.parse("""{
+            "parameters": {"base_url": "%s", "blocking_timeout": 0.1},
+            "requests": [
+                {"thread": "1", "relpath": "first"},
+                {"thread": "2", "relpath": "second"},
+                {"thread": "1", "relpath": "no-this"},
+                {"thread": "2", "relpath": "or-this"},
+                {"thread": "1", "relpath": "third"},
+                {"thread": "2", "relpath": "fourth"}
+            ]}""" % (url,))
+        script.run()
+        worker = script._get_worker("1")
+        self.assertEqual([("first", True), ('no-this', False),
+                          ("third", True)],
+                         [(s[0].rsplit('/', 1)[1], s[1])
+                          for s in worker.stats])
+        worker = script._get_worker("2")
+        self.assertEqual([("second", True), ('or-this', False),
+                          ("fourth", True)],
+                         [(s[0].rsplit('/', 1)[1], s[1])
+                          for s in worker.stats])
+
+
+class TestRunScript(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestRunScript, self).setUp()
+        self.transport_readonly_server = http_server.HttpServer
+
+    def test_run_script(self):
+        self.build_tree(['file1', 'file2', 'file3', 'file4'])
+        url = self.get_readonly_url()
+        self.build_tree_contents([('localhost.script', """{
+    "parameters": {"base_url": "%s", "blocking_timeout": 0.1},
+    "requests": [
+        {"thread": "1", "relpath": "file1"},
+        {"thread": "2", "relpath": "file2"},
+        {"thread": "1", "relpath": "file3"},
+        {"thread": "2", "relpath": "file4"}
+    ]
+}""" % (url,))])
+        script = load_test.run_script('localhost.script')
+        worker = script._threads["1"][0]
+        self.assertEqual([("file1", True), ('file3', True)],
+                         [(s[0].rsplit('/', 1)[1], s[1])
+                          for s in worker.stats])
+        worker = script._threads["2"][0]
+        self.assertEqual([("file2", True), ("file4", True)],
+                         [(s[0].rsplit('/', 1)[1], s[1])
+                          for s in worker.stats])

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2009-06-08 23:02:49 +0000
+++ loggerhead/tests/test_simple.py	2011-02-12 13:25:20 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009 Canonical Ltd.
+# Copyright (C) 2007-2011 Canonical Ltd.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,9 +17,13 @@
 
 import cgi
 import logging
+import re
 
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.util.configobj.configobj import ConfigObj
+try:
+    from bzrlib.util.configobj.configobj import ConfigObj
+except ImportError:
+    from configobj import ConfigObj
 from bzrlib import config
 
 from loggerhead.apps.branch import BranchWSGIApp
@@ -61,7 +65,7 @@
                              'with<htmlspecialchars\n')
         self.build_tree_contents(
             [('myfilename', self.filecontents)])
-        self.tree.add('myfilename')
+        self.tree.add('myfilename', 'myfile-id')
         self.fileid = self.tree.path2id('myfilename')
         self.msg = 'a very exciting commit message <'
         self.revid = self.tree.commit(message=self.msg)
@@ -71,6 +75,11 @@
         res = app.get('/changes')
         res.mustcontain(cgi.escape(self.msg))
 
+    def test_changes_for_file(self):
+        app = self.setUpLoggerhead()
+        res = app.get('/changes?filter_file_id=myfile-id')
+        res.mustcontain(cgi.escape(self.msg))
+
     def test_changes_branch_from(self):
         app = self.setUpLoggerhead(served_url="lp:loggerhead")
         res = app.get('/changes')
@@ -88,8 +97,20 @@
     def test_annotate(self):
         app = self.setUpLoggerhead()
         res = app.get('/annotate', params={'file_id': self.fileid})
+        # If pygments is installed, it inserts <span class="pyg" content into
+        # the output, to trigger highlighting. And it specifically highlights
+        # the &lt; that we are interested in seeing in the output.
+        # Without pygments we have a simple: 'with&lt;htmlspecialchars'
+        # With it, we have
+        # '<span class='pyg-n'>with</span><span class='pyg-o'>&lt;</span>'
+        # '<span class='pyg-n'>htmlspecialchars</span>
+        # So we pre-filter the body, to make sure remove spans of that type.
+        body_no_span = re.sub(r'<span class="pyg-.">', '', res.body)
+        body_no_span = body_no_span.replace('</span>', '')
         for line in self.filecontents.splitlines():
-            res.mustcontain(cgi.escape(line))
+            escaped = cgi.escape(line)
+            self.assertTrue(escaped in body_no_span,
+                            "did not find %r in %r" % (escaped, body_no_span))
 
     def test_inventory(self):
         app = self.setUpLoggerhead()

